//#define __PRECORDER__DBG


//
//
// CPRecorder
// Ported from Delphi to CPP by lallous <lallousx86@yahoo.com>
// Lots of changes has been applied to fit my needs.
//
//

// TRecorder
//
// cyamon software
// place de l'Htel-de-Ville 8
// 1040 Echallens
// Switzerland
// www.cyamon.com
// 11/99


// This library is a freeware. You may change it, use it in your applications
// at your own risks.

// The recorder is an object that allows you to record/playback mouse and
// keyboard events. The recorder is a singleton.
// on program's startup. The recorded information is saved into a linked list.

// The recorder exports the following methods:

// int State: The recorder's state (rsIdle, rsRecording or rsPlaying).

// SpeedFactor: is the factor (in %) by which the playback speed is modified.
// Values < 100 accelerate, and values > 100 slowdown.

// int DoRecord(bool bAppend);
// Starts recording. When "Append" is true the new recorded information is appended
// to information already stored in the stream. Otherwise, the local stream is
// cleared before recording.
// It returns 0 for success or any other code on failur. Check reXXXX error codes.

// int DoPlay():
// Plays the recorded information. It returns an reXXXXX error code.

// void DoStop();
// Stops recording and/or to playing

// RecordThisMessage: callback that allow you to filter recorded message. That is give you control
// to accept or reject a message before it gets recorded. By this for example you can choose not to record mouse events.
// RecordThisMessage must point to a function of the following prototype:
// bool RecordThis(EVENTMSG &Msg);
//
//
// bool load_stream(LPCTSTR FileName)
// bool save_stream(LPCTSTR)
// These methods save/load a stream from/to disk.	
//


/* ----------------------------------------------------------------------------- 
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ----------------------------------------------------------------------------- 
 */

#include "precorder.h"

std::vector<EVENTMSG>                   CPRecorder::Stream;
EVENTMSG                                CPRecorder::EventMsg;
CPRecorder::eRecorderState              CPRecorder::State = rsIdle;
long                                    CPRecorder::BaseTime;
long                                    CPRecorder::SpeedFactor = 100;
HHOOK                                   CPRecorder::HookHandle = NULL;
HINSTANCE                               CPRecorder::hInstance = GetModuleHandle(NULL);
CPRecorder::tEventMsgStream::iterator   CPRecorder::StreamPtr;


bool (*CPRecorder::RecordThisMessage)(EVENTMSG &Msg) = CPRecorder::DefaultRecordAllMessages;


//------------------------------------------------------------------------------
// Journal Playback callback procedure
//
LRESULT WINAPI CPRecorder::PlayProc(int code, WPARAM wParam, LPARAM lParam)
{
  LRESULT rc;

  _icebp
  if (code < 0)
    rc = CallNextHookEx(HookHandle, code, wParam, lParam);
  else
  {
    switch (code)
    {
      //-------------------------------------------------------
      case HC_SKIP:
        // play as long as stream is not empty!
        if (StreamPtr != Stream.end())
        {
          EventMsg = *StreamPtr++;
          EventMsg.time = (SpeedFactor * (EventMsg.time / 100)) + BaseTime;
        }
        else
          // no more to play back? simply stop playing!
          SetState(rsIdle);
        break;

      //-------------------------------------------------------      
      case HC_GETNEXT:
        rc = EventMsg.time - GetTickCount();
        if (rc < 0)
          rc = 0;
        // copy event message
        *LPEVENTMSG(lParam) = EventMsg;
        break;

      //-------------------------------------------------------      
      default:
        *LPEVENTMSG(lParam) = EventMsg;
        rc = CallNextHookEx(HookHandle, code, lParam, lParam);        
    }
  }
  return rc;
}


//------------------------------------------------------------------------------
// Journal Record callback procedure
//
LRESULT WINAPI CPRecorder::RecordProc(int code, WPARAM wParam, LPARAM lParam)
{
  LRESULT rc;

  _icebp
  if (code < 0)
    rc = CallNextHookEx(HookHandle, code, wParam, lParam);
  else
  {
    switch (code)
    {
      //-------------------------------------------------------      
      case HC_ACTION:
        EventMsg = *LPEVENTMSG(lParam);
        EventMsg.time -= BaseTime;

        // Stop recording ?
        if ( (EventMsg.message >= WM_KEYFIRST) && 
             (EventMsg.message <= WM_KEYLAST)  &&
             (EventMsg.paramL & 0xFF == VK_CANCEL)
           )
        {
          SetState(rsIdle);
        }

        if (RecordThisMessage(EventMsg))
          Stream.push_back(EventMsg);
        break;
      //-------------------------------------------------------      
      case HC_SYSMODALON:
      case HC_SYSMODALOFF:
        break;
    }
  }
  return rc;
}


//------------------------------------------------------------------------------
// Plays a recorded stream
//
int CPRecorder::DoPlay()
{
  if (State != rsIdle)
    return reNotIdle;
  else if (Stream.empty())
    return reEmpty;
  else
  {
    StreamPtr = Stream.begin();
    EventMsg = *StreamPtr++;

    // Install playback hook
    HookHandle = SetWindowsHookEx(WH_JOURNALPLAYBACK, PlayProc, hInstance, 0);

    // not successfull
    if (HookHandle == NULL)
      return rePlaybackHook;
    else
    {
      BaseTime = GetTickCount();
      SetState(rsPlaying);
    }
  }
  return reOk;
}


//------------------------------------------------------------------------------
// Allows you to record events.
// If bAppend then we will append new recording into previous recording
//
int CPRecorder::DoRecord(bool bAppend)
{
  if (State != rsIdle)
    return reNotIdle;
  else
  {
    // no append ? clear Stream and re-init base time
    if (!bAppend)
    {
      BaseTime = GetTickCount();
      Stream.clear();
    }
    else
    {
      EventMsg.time = 0;
      // stream not empty ?
      if (!Stream.empty())
      {
        EventMsg = Stream.back();
      }
      BaseTime = GetTickCount();
    }
    //
    HookHandle = SetWindowsHookEx(WH_JOURNALRECORD, RecordProc, hInstance, 0);
    if (HookHandle == NULL)
      return reRecorderHook;
    else
      SetState(rsRecording);
  }
  return reOk;
}


//------------------------------------------------------------------------------
// Stops recording
//
void CPRecorder::DoStop()
{
  SetState(rsIdle);
}


//------------------------------------------------------------------------------
// Changes the state of the recorder
// If it was recording then it stops the recorder and removes the journal rec. hook
//
void CPRecorder::SetState(eRecorderState newState)
{
  // user wants us to become idle ?
  if ( (newState == rsIdle) && (HookHandle != NULL))
  {
    UnhookWindowsHookEx(HookHandle);
    HookHandle = NULL;
  }
  State = newState;
}

//------------------------------------------------------------------------------
// Default callback that will capture all messages by default
//
bool CPRecorder::DefaultRecordAllMessages(EVENTMSG &msg)
{
  return true;
}


//------------------------------------------------------------------------------
// Saves a stream into a disk file
//
bool CPRecorder::save_stream(LPCTSTR FileName)
{
  FILE *fp = fopen(FileName, "wb");
  if (!fp)
    return false;

  int c = Stream.size();

  StreamPtr = Stream.begin();

  for (int i=0;i<c;i++)
    fwrite((void *)&(*StreamPtr++), sizeof(EVENTMSG), 1, fp);

  fclose(fp);
  return true;
}


//------------------------------------------------------------------------------
// Loads a stream from disk into memory
// It will be ready to be played w/ the DoPlay()
bool CPRecorder::load_stream(LPCTSTR FileName)
{
  FILE *fp = fopen(FileName, "rb");
  if (!fp)
    return false;
  
  EVENTMSG e;

  Stream.clear();
  while (!feof(fp))
  {
    fread((void *)&e, sizeof(EVENTMSG), 1, fp);
    Stream.push_back(e);
  }
  fclose(fp);
  return true;
}

//------------------------------------------------------------------------------
// Constructor
//
CPRecorder::CPRecorder()
{
  Stream.clear();
  State = rsIdle;
  HookHandle = NULL;
}