// idamac.cpp : Defines the initialization routines for the DLL.
//

#include "stdafx.h"
#include "idamac.h"
#include <ida.hpp>
#include <idp.hpp>
#include <loader.hpp>
#include <kernwin.hpp>
#include <PRecorder.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only CIdamacApp object

CIdamacApp theApp;

/////////////////////////////////////////////////////////////////////////////
// The one and only CPRecorder object

CPRecorder rec;


bool show_playing_msg = false;

//
//	Note!
//
//		If this DLL is dynamically linked against the MFC
//		DLLs, any functions exported from this DLL which
//		call into MFC must have the AFX_MANAGE_STATE macro
//		added at the very beginning of the function.
//
//		For example:
//
//		extern "C" BOOL PASCAL EXPORT ExportedFunction()
//		{
//			AFX_MANAGE_STATE(AfxGetStaticModuleState());
//			// normal function body here
//		}
//
//		It is very important that this macro appear in each
//		function, prior to any calls into MFC.  This means that
//		it must appear as the first statement within the 
//		function, even before any object variable declarations
//		as their constructors may generate calls into the MFC
//		DLL.
//
//		Please see MFC Technical Notes 33 and 58 for additional
//		details.
//

/////////////////////////////////////////////////////////////////////////////
// CIdamacApp

BEGIN_MESSAGE_MAP(CIdamacApp, CWinApp)
	//{{AFX_MSG_MAP(CIdamacApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIdamacApp construction

CIdamacApp::CIdamacApp()
{
  rec.SpeedFactor = 1; // fastest playback possible!
  rec.RecordThisMessage = RecordThisMessage;
}



//--------------------------------------------------------------------------
// Recorder filter playback
// This function allows you to filter messages that you want to record or not
bool RecordThisMessage(EVENTMSG &eventmsg)
{
  // only capture keyboard events
  if ((eventmsg.message >= WM_KEYFIRST) && (eventmsg.message <= WM_KEYLAST))
    return true;
  return false;
}



//--------------------------------------------------------------------------
//
//      Initialize.
//
//      IDA will call this function only once.
//      If this function returns PLGUIN_SKIP, IDA will never load it again.
//      If this function returns PLUGIN_OK, IDA will unload the plugin but
//      remember that the plugin agreed to work with the database.
//      The plugin will be loaded again if the user invokes it by
//      pressing the hotkey or selecting it from the menu.
//      After the second load the plugin will stay on memory.
//      If this function returns PLUGIN_KEEP, IDA will keep the plugin
//      in the memory. In this case the initialization function can hook
//      into the processor module and user interface notification points.
//      See the hook_to_notification_point() function.
//
//      In this example we check the input file format and make the decision.
//      You may or may not check any other conditions to decide what you do:
//      whether you agree to work with the database or not.
//
int init(void)
{
  msg("IDAMAC plugin v0.1\n");
  return PLUGIN_KEEP;
}



//--------------------------------------------------------------------------
//      Terminate.
//      Usually this callback is empty.
//      The plugin should unhook from the notification lists if
//      hook_to_notification_point() was used.
//
//      IDA will call this function when the user asks to exit.
//      This function won't be called in the case of emergency exits.
void term(void)
{
  rec.DoStop();
}



void play()
{
  if (show_playing_msg)
  {
    msg("playing...\n");
    show_playing_msg = false;
  }
  rec.DoPlay();
}


void stop()
{
  if (rec.State == CPRecorder::rsRecording)
  {
    rec.DoStop();	
    msg("stopped recording...\n");
    show_playing_msg = true;
  }
}

void record()
{
  if (rec.State != CPRecorder::rsRecording)
  {
    rec.DoRecord();
    msg("started recording\n");
  } else msg("already recording!\n");
}



//--------------------------------------------------------------------------
//
//      The plugin method
//
//      This is the main function of plugin.
//
//      It will be called when the user selects the plugin.
//
//              arg - the input argument, it can be specified in
//                    plugins.cfg file. The default is zero.
//
//
void run(int arg)
{
  switch (arg)
  {
  case 0:
    record();
    break;
  case 1:
    stop();
    break;
  case 2:
    play();
    break;
  }
}

//--------------------------------------------------------------------------
char comment[] = "This is simple plugin that will allow keyboard macros in IDAGUI";

char help[] =
        "This plugin is created by lallous\n";



// This is the preferred hotkey for the plugin module
// The preferred hotkey may be overriden in plugins.cfg file
// Note: IDA won't tell you if the hotkey is not correct
//       It will just disable the hotkey.


//--------------------------------------------------------------------------
//
//      PLUGIN DESCRIPTION BLOCK
//
//--------------------------------------------------------------------------

extern "C" plugin_t PLUGIN = {
  IDP_INTERFACE_VERSION,
  0,                    // plugin flags
  init,                 // initialize
  term,                 // terminate. this pointer may be NULL.
  run,                  // invoke plugin
  comment,              // long comment about the plugin
                        // it could appear in the status line
                        // or as a hint
  help,                 // multiline help about the plugin
  "Start Recording",    // the preferred short name of the plugin
  "Alt-="               // the preferred hotkey to run the plugin
};